#
# Mandatory for openpluginlib initialisation
#

import platform
import sys
import os.path

# This is required for the executable generated from py2exe
# to use the preinstalled openlibraries python bindings
if platform.system() == "Windows":
	sys.path.append(os.environ['PYTHONPATH'])

# Initialise the dynamic linking functionality - note that the ubuntu amd64
# distro lacks the dl module, hence fallback to the non-symbolic values
elif platform.system( ) == "Linux":
	try:
		import dl
		sys.setdlopenflags( dl.RTLD_NOW | dl.RTLD_GLOBAL )
	except:
		sys.setdlopenflags( 257 )

# Initialise openpluginlib
import openpluginlib

# Darwin initialisation with and without jahwidgets (jbatch and associated
# tools don't have a dependency on jahwidgets directly)
if platform.system() == "Darwin":
	# Add search paths
	try:
		import jahwidgets
		olibs = jahwidgets.get_bundle_resources_directory_path()
		olibs += os.path.join( "/../Frameworks/openlibraries-0.4.0" )

		plugin_paths = [ olibs + "/openassetlib/plugins", 
						 olibs + "/openeffectslib/plugins",
						 olibs + "/openimagelib/plugins",
						 olibs + "/openmedialib/plugins",
						 olibs + "/openobjectlib/plugins" ]

		openpluginlib.init( plugin_paths )
	except ImportError:
		openpluginlib.init()
else:
	openpluginlib.init()

# Register custom stream handlers

import handlers
import openimagelib
import openmedialib

openmedialib.stream_handler_register( handlers.url_callback )

