#!/usr/bin/env python

import threading
import socket
import sys
import urllib
import os.path

def execute( commands, host = "localhost", port = 45537 ):
	result = "OK"
	if len( commands ) > 0:
		cmd = ""
		for i in commands:
			if os.path.exists( i ):
				i = os.path.abspath( i )
			cmd += urllib.quote( i ) + " "
		s = socket.socket( socket.AF_INET, socket.SOCK_STREAM )
		s.connect( ( host, port ) )
		s.send( "text: " + str( len( cmd ) ) + "\n" )
		s.send( cmd )
		result = s.recv( 1024 )
		s.close( )
	return result

class status( threading.Thread ):
	def __init__( self, status_cb, host = "localhost", port = 45537 ):
		threading.Thread.__init__( self )
		self.setDaemon( True )
		self.status_cb = status_cb
		self.host = host
		self.port = port
		self.s = socket.socket( socket.AF_INET, socket.SOCK_STREAM )
		self.s.connect( ( self.host, self.port ) )
		self.start( )
	
	def run( self ):
		try:
			self.s.send( "status\n" )
			data = self.s.recv( 1024 )
			while data != "" and data != "OK":
				stuff = data.split( " " )
				if len( stuff ) == 2:
					self.status_cb( int( stuff[ 0 ] ), int( stuff[ 1 ] ) )
				data = self.s.recv( 1024 )
			self.s.close( )
		except Exception, e:
			self.status_cb( -1, -1 )

def usage( ):
	print """
jbatch_client [ options ] [ op ]*

Options:

    -h, --host=host : hostname to access (default: localhost)
    -p, --port=port : port number to use (default: 45537)
    -v, --verbose   : display return from the server

Special:

    --help          : this information

Examples:

    To queue a media file on the batch server:
    $ jbatch_client <media>

    To start playout:
    $ jbatch_client start

    To force playout on the next pending media:
    $ jbatch_client stop start

    To clear the pending media:
    $ jbatch_client clear

    To shutdown the server:
    $ jbatch_client shutdown

Additional:

    See jbatch_server.
"""

def main( ):
	ops = [ ]
	switches = { }
	switches[ "host" ] = "localhost"
	switches[ "port" ] = "45537"
	verbose = False
	next = None

	for i in sys.argv[ 1: ]:
		arg = None

		if i.startswith( "--" ) and i.find( "=" ) > 0:
			i, arg = i.split( "=", 1 )
		if i in ( "-p", "--port" ):
			next = "port"
		elif i in ( "-h", "--host" ):
			next = "host"
		elif i in ( "-v", "--verbose" ):
			verbose = True
		elif i in ( "--help" ):
			usage( )
			sys.exit( 0 )
		else:
			arg = i

		if arg is not None:
			if next is not None:
				switches[ next ] = arg
				next = None
			else:
				ops.append( arg )

	if len( ops ) > 0:
		try:
			result = execute( ops, switches[ "host" ], int( switches[ "port" ] ) )
			if verbose:
				print result
		except socket.error:
			print "Connection to " + switches[ "host" ] + ":" + switches[ "port" ] + " failed."
			sys.exit( 1 )
	else:
		usage( )

if __name__ == "__main__":
	main( )

