#!/usr/bin/env python

import bootstrap
import openpluginlib
import openimagelib
import openmedialib
import sys
import jstack
import socket
import urllib
import threading
import common
import balance
import time
import os

class rpn_thread( threading.Thread ):
	output = None
	store = None
	store_properties = [ ]
	conform = [ ]
	rpn = [ ]

	def __init__( self ):
		threading.Thread.__init__( self )
		self.queue = [ ]
		self.rpn = jstack.stack( )
		self.rpn.dict[ "start" ] = self.execute
		self.rpn.push( "jstack.oml" )
		self.mutex = threading.Lock( )
		self.not_empty = threading.Condition( self.mutex )
		self.notify = threading.Lock( )
		self.notification = threading.Condition( self.notify )
		self.running = True
		self.position = 0
		self.length = 0
		self.speed = 1
		self.playing = False
		self.dict = { }
		self.dict[ "shutdown" ] = self.shutdown
		self.dict[ "play" ] = self.play
		self.dict[ "pause" ] = self.pause
		self.dict[ "stop" ] = self.stop
		self.dict[ "ffwd" ] = self.ffwd
		self.dict[ "rew" ] = self.rew
		self.dict[ "seek" ] = self.seek

	def run( self ):
		self.rpn.push_args( rpn_thread.rpn )
		while self.running:
			self.not_empty.acquire( )
			while self.running and len( self.queue ) > 0:
				arg = self.queue.pop( 0 )
				self.not_empty.release( )
				self.rpn.push( arg )
				self.not_empty.acquire( )
			if self.running:
				self.not_empty.wait( )
			self.not_empty.release( )

	def push( self, arg ):
		self.not_empty.acquire( )
		if arg in self.dict.keys( ):
			self.dict[ arg ]( )
		else:
			self.queue.append( arg )
		self.not_empty.notifyAll( )
		self.not_empty.release( )
		return self.running

	def shutdown( self ):
		self.playing = False
		self.running = False
		self.notification.acquire( )
		self.notification.notifyAll( )
		self.notification.release( )

	def play( self, speed = 1 ):
		self.speed = speed
		if not self.playing and speed != 0:
			self.queue.append( "start" )

	def stop( self ):
		self.playing = False

	def pause( self ):
		self.speed = 0

	def ffwd( self ):
		self.play( 25 )

	def rew( self ):
		self.play( -25 )

	def seek( self ):
		tos = self.queue.pop( )
		self.position = int( tos )
		self.play( )

	def execute( self ):
		if not self.playing and not self.rpn.empty( ):
			input = self.rpn.pop( )
			if input is not None:
				self.rpn.push_input( input )
				self.rpn.push_args( rpn_thread.conform )
				modified = self.rpn.pop( )
				self.preview( modified )
				if not self.playing or ( modified.get_position( ) >= modified.get_frames( ) - 1 and not self.rpn.empty( ) ):
					input.seek( 0, False )
					self.rpn.deposit( modified )
				else:
					self.rpn.push_input( input )
				self.playing = False

	def preview( self, input ):
		output = None
		frame = input.fetch( )
		if frame is not None:
			self.playing = True

			if rpn_thread.output is None:
				if rpn_thread.store is None:
					if len( rpn_thread.store_properties ) > 0:
						rpn_thread.store = rpn_thread.store_properties.pop( 0 )

			if rpn_thread.store is None:
				output = common.preview( frame )
			elif rpn_thread.output is not None:
				output = rpn_thread.output
			else:
				self.rpn.push( "op:" + str( rpn_thread.store ) )
				self.rpn.push_args( rpn_thread.store_properties )
				op = self.rpn.pop( )
				rpn_thread.output = openmedialib.create_store( str( op.uri ), frame )
				common.assign_props( rpn_thread.output.properties( ), op.operand.properties )
				rpn_thread.output.init( )
				output = rpn_thread.output
			balance.activate( input )
			self.position = input.get_position( )
			self.length = input.get_frames( )
			while self.playing and self.running and self.position < input.get_frames( ):
				self.not_empty.acquire( )
				speed = self.speed
				self.not_empty.release( )
				if not output.push( frame ): break
				self.notification.acquire( )
				self.notification.notifyAll( )
				self.notification.release( )
				if self.position != input.get_position( ): output.flush( )
				self.position += speed
				if self.position < 0: self.position = 0
				input.seek( self.position, False )
				frame = input.fetch( )
				while self.running and self.speed == 0 and self.position == input.get_position( ):
					self.not_empty.acquire( )
					self.not_empty.wait( )
					self.not_empty.release( )
				time.sleep( 0.0001 )
			balance.deactivate( input )
			output.flush( )

class rpn_status( threading.Thread ):
	def __init__( self, server, client, address ):
		threading.Thread.__init__( self )
		self.setDaemon( True )
		self.server = server
		self.client = client
		self.address = address

	def run( self ):
		connected = True
		while self.server.rpn.running and connected:
			output = "%d %d\n" % ( self.server.rpn.position, self.server.rpn.length )
			connected = self.client.send( output ) == len( output )
			if connected:
				self.server.rpn.notification.acquire( )
				if self.server.rpn.running:
					self.server.rpn.notification.wait( )
				self.server.rpn.notification.release( )
		if connected:
			self.client.send( "OK" )

class rpn_server( socket.socket ):
	def __init__( self, info ):
		socket.socket.__init__( self, socket.AF_INET, socket.SOCK_STREAM )
		self.setsockopt( socket.SOL_SOCKET, socket.SO_REUSEADDR, 1 )
		self.info = info
		self.finished = False
		self.rpn = None

	def run( self ):
		self.finished = False

		try:
			self.bind( self.info )
			self.listen( 5 )
			self.rpn = rpn_thread( )
			self.rpn.start( )

			while not self.finished:
				client, address = self.accept( )
				self.finished = self.handle( client, address )
		finally:
			self.close( )
			if self.rpn is not None:
				self.rpn.join( )

	def status( self, client, address ):
		rpn_status( self, client, address ).start( )

	def handle( self, client, address ):
		finished = False
		cmd = client.recv( 4096 )
		if cmd.find( "status" ) == 0:
			self.status( client, address )
			return finished
		if cmd.find( "text: " ) == 0:
			parts = cmd.split( "\n" )
			length = int( parts[ 0 ][ 6: ].strip( ) )
			if len( parts ) > 1:
				cmd = cmd[ cmd.find( "\n" ) + 1: ]
				length -= len( cmd )
			else:
				cmd = ""
			while length > 0:
				packet = client.recv( 4096 )
				length -= len( packet )
				cmd += packet
		lines = cmd.split( "\n" )
		for line in lines:
			args = line.split( " " )
			for arg in args:
				finished = not self.rpn.push( urllib.unquote( arg ) )
				if finished: break
			if finished: break
		client.send( "OK" )
		client.close( )
		return finished

def usage( ):
	print """
jbatch_server [ options ]

Options:

    -h, --host=host       : hostname to limit access (default: localhost)
    -p, --port=port       : port number to use (default: 45537)
    -r, --rpn=[ op ]*     : initial content for rpn stack 
    -c, --conform=[ op ]* : rpn ops to apply to all media
    -s, --store=[ op ]*   : rpn ops to define the store

Special:

    -v, --verbose         : verbose diagnostics (default: off)
    -d, --debug           : don't fork
    --help                : this information

Examples:

    For a simple playout server:
    $ jbatch_server

    For a normalised playout server:
    $ jbatch_server --rpn pal-vcd.oml --conform normalise

    For an audio only playout server:
    $ jbatch_server --conform filter:conform image=0 audio=1 filter:resampler --store openal:

    For a PAL vcd mpeg feed to an oml socket:
    $ jbatch_server --rpn pal-vcd.oml --conform normalise --store oml:tcp:.mpg encode_flags

    For a PAL vcd mpeg feed with a jcapture web front end:
    $ jbatch_server --rpn pal-vcd.oml --conform normalise --store oml:tcp:.mpg encode_flags
    $ jcapture --device=oml:tcp:.mpg --block=1024 --extension=render
    $ jbatch_client <media> start
    $ jbatch http://localhost:51030/jah.mpg

Additional:

    All jbatch_server instances can be controlled via jbatch_client and jbatch_ui.
"""

def main( ):
	props = rpn_thread.store_properties
	switches = { }
	switches[ "fork" ] = hasattr( os, "fork" )
	switches[ "host" ] = "localhost"
	switches[ "port" ] = "45537"
	next = None
	balance.verbose = False

	for i in sys.argv[ 1: ]:
		arg = None

		if i.startswith( "--" ) and i.find( "=" ) > 0:
			i, arg = i.split( "=", 1 )
		if i in ( "-p", "--port" ):
			next = "port"
		elif i in ( "-h", "--host" ):
			next = "host"
		elif i in ( "-c", "--conform" ):
			props = rpn_thread.conform
		elif i in ( "-s", "--store" ):
			props = rpn_thread.store_properties
		elif i in ( "-r", "--rpn" ):
			props = rpn_thread.rpn
		elif i in ( "-d", "--debug" ):
			switches[ "fork" ] = False
		elif i in ( "-v", "--verbose" ):
			balance.verbose = True
		elif i in ( "-h", "--help" ):
			usage( )
			sys.exit( 0 )
		else:
			arg = i

		if arg is not None:
			if next is not None:
				switches[ next ] = arg
				next = None
			else:
				props.append( arg )

	try:
		server = rpn_server( ( switches[ "host" ], int( switches[ "port" ] ) ) )
		if switches[ "fork" ]:
			pid = os.fork( )
			if pid != 0:
				sys.exit( 0 )
		server.run( )
	except socket.error, e:
		print "Server already running - aborting."

if __name__ == "__main__":
	main( )
