#!/usr/bin/env python

import bootstrap
import sys
import openpluginlib
import openimagelib
import openmedialib
import jstack
import keygrab
import Tkinter
import tkFileDialog
import common
import balance
import time
import os
import threading
import urlparse
import urllib

def usage( ):
	print "jbatch_tk [ <media> ]*"

class App( Tkinter.Frame ):
	def __init__( self, master, ops ):
		Tkinter.Frame.__init__( self, master )

		self.running = True
		self.master = master
		self.width = 640
		self.height = 480
		self.speed = 1
		self.visible_tran = True
		self.visible_list = True
		self.input = None
		self.output = None
		self.ops = ops
		self.playing = 0
		self.request = 0
		self.visualise = None
		self.visualise_force = 0
		self.jcapture_mode = False
		self.jcapture_server = ""

		master.title( "jbatch" )

		container = Tkinter.Frame( master )
		container.pack( expand = True, anchor = Tkinter.W, fill = Tkinter.BOTH )

		frame = Tkinter.Frame( container, width = self.width, height = self.height, bg = "", colormap = "new" )
		frame.bind( "<Configure>", self.resize )
		frame.bind( "<KeyRelease-Escape>", self.exit )
		frame.bind( "<KeyRelease-f>", self.full_screen )
		frame.bind( "<KeyRelease-h>", self.hide_transport )
		frame.bind( "<KeyRelease-l>", self.hide_list )
		frame.bind( "<KeyRelease-space>", self.toggle )
		frame.bind( "<Button-1>", self.seek )
		frame.focus_force( )
		frame.pack( side = Tkinter.RIGHT, expand = True, fill = Tkinter.BOTH )

		self.list = Tkinter.Listbox( container )
		self.list.bind( "<ButtonRelease-1>", self.select )
		self.list.pack( anchor = Tkinter.W, side = Tkinter.LEFT, fill = Tkinter.BOTH )

		if len( ops ) == 1:
			self.hide_list( )

		self.rpn = jstack.stack( )
		self.rpn.push( "jstack.oml" )
		self.rpn.push( "sdl.winid=%d" % frame.winfo_id( ) )
		self.rpn.push( "sdl.box=letter" )

		frame = Tkinter.Frame( master )
		frame.pack( fill = Tkinter.BOTH )
		self.transport_frame = frame

		self.b_open = Tkinter.Button( frame, text = "Open", command = self.open )
		self.b_open.pack( side = Tkinter.LEFT )

		self.b_rew = Tkinter.Button( frame, text = "<<" )
		self.b_rew.bind( "<Button-1>", self.rew )
		self.b_rew.bind( "<ButtonRelease-1>", self.restart )
		self.b_rew.pack( side = Tkinter.LEFT )
		self.b_prev = Tkinter.Button( frame, text = "|<" )
		self.b_prev.bind( "<Button-1>", self.prev )
		self.b_prev.pack( side = Tkinter.LEFT )
		self.v_pause = Tkinter.StringVar( frame )
		self.v_pause.set( "||" )
		self.b_pause = Tkinter.Button( frame, textvariable = self.v_pause, command = self.toggle )
		self.b_pause.pack( side = Tkinter.LEFT )
		self.b_next = Tkinter.Button( frame, text = ">|" )
		self.b_next.bind( "<Button-1>", self.next )
		self.b_next.pack( side = Tkinter.LEFT )
		self.b_ffwd = Tkinter.Button( frame, text = ">>" )
		self.b_ffwd.bind( "<Button-1>", self.ffwd )
		self.b_ffwd.bind( "<ButtonRelease-1>", self.restart )
		self.b_ffwd.pack( side = Tkinter.LEFT )

		self.b_scale_pressed = False
		self.b_scale = Tkinter.Scale( frame, orient = Tkinter.HORIZONTAL, showvalue = False, repeatdelay = 1000, command = self.scrub )
		self.b_scale.bind( "<Button-1>", self.scale_down )
		self.b_scale.bind( "<ButtonRelease-1>", self.scale_up )
		self.b_scale.pack( side = Tkinter.LEFT, expand = True, fill = Tkinter.BOTH )

		self.box = Tkinter.StringVar( frame )
		self.box.set( "letter" )
		Tkinter.Radiobutton( frame, text = "Letter", variable = self.box, value = "letter", command = self.box_callback ).pack( side = Tkinter.LEFT )
		Tkinter.Radiobutton( frame, text = "Pillar", variable = self.box, value = "pillar", command = self.box_callback ).pack( side = Tkinter.LEFT )

		Tkinter.Checkbutton( frame, text = "Visualise", command = self.visualise_callback ).pack( side = Tkinter.LEFT )

		if len( ops ) > 0:
			for i in ops:
				path, name = os.path.split( i )
				self.list.insert( Tkinter.END, name )

	def run( self ):
		while self.running:
			try:
				if self.request >= len( self.ops ):
					self.request = 0
				if self.request < len( self.ops ):
					self.jcapture_mode = self.ops[ self.request ].startswith( "oml:http://" ) and self.ops[ self.request ].endswith( "/jah.dv" )
					if self.jcapture_mode:
						self.jcapture_server = urlparse.urlsplit( self.ops[ self.request ][ 4: ] )[ 1 ]
						self.speed = 1
						self.pause( )
					self.rpn.push( self.ops[ self.request ] )
					if self.jcapture_mode:
						self.rpn.push_args( [ "audio_index=-1" ] )
				else:
					self.rpn.push_args( [ "test:" ] )
				self.input = self.rpn.pop( )
				if self.input is not None and self.input.get_frames( ) > 0:
					self.playing = self.request
					self.list.select_clear( 0, self.list.size( ) )
					self.list.select_set( self.request )
					self.rpn.push_input( self.input )
					self.rpn.push_args( [ "filter:resampler" ] )
					self.visualise = openmedialib.create_filter( "visualise" )
					self.rpn.push_input( self.visualise )
					self.rpn.push_args( [ "colourspace=yuv420p", "width=320", "height=240", "force=%d" % self.visualise_force ] )
					self.rpn.push_args( [ "filter:conform" ] )
					if self.jcapture_mode: self.rpn.push_args( [ "audio=0" ] )
					self.rpn.push( "filter:deinterlace" )
					self.input = self.rpn.pop( )
					self.b_scale.config( from_ = 0, to = self.input.get_frames( ) )
					self.b_scale.set( 0 )
					self.play( self.input )
					if self.jcapture_mode:
						self.pause( )
					self.jcapture_mode = False
				else:
					self.list.delete( self.request )
					self.ops.pop( self.request )
					self.playing = self.request - 1
				if self.playing == self.request:
					self.request += 1
			except jstack.stack_exception, e:
				self.list.delete( self.request )
				self.ops.pop( self.request )
				self.playing = self.request - 1

	def open( self ):
		files = tkFileDialog.askopenfilenames( parent = self.master )
		self.ops += files
		if len( files ) > 0:
			for i in files:
				path, name = os.path.split( i )
				self.list.insert( Tkinter.END, name )

	def prev( self, event = None ):
		if self.jcapture_mode:
			self.pause( )
			self.stream = urllib.urlopen( "http://" + self.jcapture_server + "/jah/prev" )
			self.stream.read( )

	def rew( self, event ):
		if not self.jcapture_mode:
			self.speed = -25
		else:
			self.stream = urllib.urlopen( "http://" + self.jcapture_server + "/jah/rewind" )
			self.stream.read( )

	def start( self, event = None ):
		if not self.jcapture_mode:
			self.speed = 1
			self.v_pause.set( "||" )
		else:
			self.stream = urllib.urlopen( "http://" + self.jcapture_server + "/jah/play" )
			self.stream.read( )

	def restart( self, event = None ):
		if not self.jcapture_mode:
			self.start( )
		else:
			self.stream = urllib.urlopen( "http://" + self.jcapture_server + "/jah/pause" )
			self.stream.read( )

	def stop( self, event = None ):
		if not self.jcapture_mode:
			self.speed = 0
			self.v_pause.set( "||" )
		else:
			self.stream = urllib.urlopen( "http://" + self.jcapture_server + "/jah/stop" )
			self.stream.read( )

	def pause( self, event = None ):
		if not self.jcapture_mode:
			self.speed = 0
			self.v_pause.set( ">" )
		else:
			self.stream = urllib.urlopen( "http://" + self.jcapture_server + "/jah/pause" )
			self.stream.read( )

	def toggle( self, event = None ):
		if not self.jcapture_mode:
			if self.speed != 0:
				self.speed = 0
				self.v_pause.set( ">" )
			else:
				self.speed = 1
				self.v_pause.set( "||" )
		else:
			self.stream = urllib.urlopen( "http://" + self.jcapture_server + "/jah/toggle" )
			self.stream.read( )

	def ffwd( self, event ):
		if not self.jcapture_mode:
			self.speed = 25
		else:
			self.stream = urllib.urlopen( "http://" + self.jcapture_server + "/jah/ff" )
			self.stream.read( )

	def next( self, event = None ):
		if self.jcapture_mode:
			self.pause( )
			self.stream = urllib.urlopen( "http://" + self.jcapture_server + "/jah/next" )
			self.stream.read( )

	def play( self, input ):
		frame = input.fetch( )
		if frame is not None:
			output = common.preview( frame )
			balance.activate( input )
			i = 0
			while self.running and frame is not None and i < input.get_frames( ):
				if self.request != self.playing: break
				if not output.push( frame ): break
				expected = input.get_position( )
				self.update( frame )
				i = input.get_position( )
				if i != expected or self.speed != 1:
					output.flush( )
				if self.speed != 0:
					i += self.speed
					if i < 0: i = 0
					input.seek( i, False )
					frame = input.fetch( )
				else:
					frame.set_audio( None )
					time.sleep( .2 )
			balance.deactivate( input )
			if output is not None:
				output.complete( )
		else:
			print "no frame"

	def update( self, event ):
		if not self.b_scale_pressed:
			self.b_scale.set( event.get_position( ) )
		self.master.update( )

	def resize( self, event ):
		self.width = event.width
		self.height = event.height
		if common.preview.video_store is not None:
			common.preview.video_store.properties( ).get_property( "width" ).set( event.width )
			common.preview.video_store.properties( ).get_property( "height" ).set( event.height )

	def box_callback( self ):
		if common.preview.video_store is not None:
			common.preview.video_store.properties( ).get_property( "box" ).set( self.box.get( ) )

	def seek( self, event ):
		if not self.jcapture_mode and event.y > int( 0.9 * self.height ):
			position = ( float( event.x ) / float( self.width ) ) * float( self.input.get_frames( ) )
			self.input.seek( int( position ), False )
			self.start( )

	def scrub( self, event ):
		if not self.jcapture_mode and self.b_scale_pressed:
			self.input.seek( int( event ), False )
			self.start( )

	def scale_up( self, event ):
		self.b_scale_pressed = False

	def scale_down( self, event ):
		self.b_scale_pressed = True

	def exit( self, event ):
		self.running = False

	def full_screen( self, event ):
		w, h = self.master.winfo_screenwidth(), self.master.winfo_screenheight()
		self.master.geometry( "%dx%d%+d%+d" % ( w, h, 0, 0 ) )
		#self.master.overrideredirect( 1 )

	def hide_transport( self, event ):
		if self.visible_tran:
			self.transport_frame.pack_forget( )
		else:
			self.transport_frame.pack( fill = Tkinter.BOTH )
		self.visible_tran = not self.visible_tran

	def hide_list( self, event = None ):
		if self.visible_list:
			self.list.pack_forget( )
		else:
			self.list.pack( anchor = Tkinter.W, side = Tkinter.LEFT, fill = Tkinter.BOTH )
		self.visible_list = not self.visible_list

	def select( self, event ):
		self.request = int( self.list.curselection( )[ 0 ] )

	def visualise_callback( self ):
		if self.visualise_force == 0:
			self.visualise_force = 2
		else:
			self.visualise_force = 0
		if self.visualise:
			self.visualise.properties( ).get_property( "force" ).set( self.visualise_force )

def main( ):
	ops = [ ]

	for i in sys.argv[ 1: ]:
		arg = None

		if i.startswith( "--" ) and i.find( "=" ) > 0:
			i, arg = i.split( "=", 1 )

		if i in ( "--help" ):
			usage( )
			sys.exit( 0 )
		else:
			arg = i

		if arg is not None:
			ops.append( arg )

	root = Tkinter.Tk( )
	app = App( root, ops )
	app.run( )

	openpluginlib.uninit( )

if __name__ == "__main__":
	main( )

