#!/usr/bin/env python

import Tkinter
import tkFileDialog
import tkMessageBox
import jbatch_client
import sys
import socket

# Simple test case app for the jbatch server.

class App( Tkinter.Frame ):
	def __init__( self, master, host, port ):
		self.host = host
		self.port = port

		Tkinter.Frame.__init__( self, master )
		master.title( "jbatch ui" )

		self.current_files = [ ]

		frame = Tkinter.Frame( master )
		frame.pack( expand = False )

		self.b_rew = Tkinter.Button( frame, text = "<<" )
		self.b_rew.bind( "<Button-1>", self.rew )
		self.b_rew.bind( "<ButtonRelease-1>", self.play )
		self.b_rew.pack( side = Tkinter.LEFT )
		self.b_play = Tkinter.Button( frame, text = ">", command = self.play )
		self.b_play.pack( side = Tkinter.LEFT )
		self.b_stop = Tkinter.Button( frame, text = "[]", command = self.stop )
		self.b_stop.pack( side = Tkinter.LEFT )
		self.b_pause = Tkinter.Button( frame, text = "||", command = self.pause )
		self.b_pause.pack( side = Tkinter.LEFT )
		self.b_ffwd = Tkinter.Button( frame, text = ">>" )
		self.b_ffwd.bind( "<Button-1>", self.ffwd )
		self.b_ffwd.bind( "<ButtonRelease-1>", self.play )
		self.b_ffwd.pack( side = Tkinter.LEFT )

		frame = Tkinter.Frame( master )
		frame.pack( expand = False )

		self.b_load = Tkinter.Button( frame, text = "Load", command = self.load )
		self.b_load.pack( side = Tkinter.LEFT )
		self.b_append = Tkinter.Button( frame, text = "Add", command = self.append )
		self.b_append.pack( side = Tkinter.LEFT )
		self.b_next = Tkinter.Button( frame, text = "Next", command = self.next )
		self.b_next.pack( side = Tkinter.LEFT )
		self.b_shutdown = Tkinter.Button( frame, text = "Kill", command = self.shutdown )
		self.b_shutdown.pack( side = Tkinter.LEFT )

		frame = Tkinter.Frame( master )
		frame.pack( expand = False )

		self.v_reverse = Tkinter.IntVar( )
		Tkinter.Checkbutton( frame, text = "Reverse", variable = self.v_reverse ).grid( row = 0, column = 0, sticky = Tkinter.W )
		self.v_deinterlace = Tkinter.IntVar( )
		Tkinter.Checkbutton( frame, text = "Deinterlace", variable = self.v_deinterlace ).grid( row = 0, column = 1, sticky = Tkinter.W )
		self.v_visualise = Tkinter.IntVar( )
		Tkinter.Checkbutton( frame, text = "Visualise", variable = self.v_visualise ).grid( row = 1, column = 0, sticky = Tkinter.W )
		self.v_greyscale = Tkinter.IntVar( )
		Tkinter.Checkbutton( frame, text = "Greyscale", variable = self.v_greyscale ).grid( row = 1, column = 1, sticky = Tkinter.W )
		self.v_thread = Tkinter.IntVar( )
		Tkinter.Checkbutton( frame, text = "Thread", variable = self.v_thread ).grid( row = 2, column = 0, sticky = Tkinter.W )

		frame = Tkinter.Frame( master )
		frame.pack( expand = True, fill = "x" )

		self.b_scale_pressed = False
		self.b_scale = Tkinter.Scale( frame, orient = Tkinter.HORIZONTAL, showvalue = False, repeatdelay = 1000, command = self.scrub )
		self.b_scale.bind( "<Button-1>", self.scale_down )
		self.b_scale.bind( "<ButtonRelease-1>", self.scale_up )
		self.b_scale.pack( side = Tkinter.LEFT, expand = True, fill = "x" )

		jbatch_client.status( self.status_cb, self.host, self.port )

	def execute( self, commands ):
		try:
			jbatch_client.execute( commands, self.host, self.port )
		except socket.error:
			tkMessageBox.showerror( "jbatch_ui", "Server communication dropped - client exiting" )
			sys.exit( 0 )

	def play( self, event = None ):
		self.execute( [ "play" ] )

	def pause( self ):
		self.execute( [ "pause" ] )

	def stop( self ):
		self.execute( [ "stop" ] )

	def ffwd( self, event ):
		self.execute( [ "ffwd" ] )

	def rew( self, event ):
		self.execute( [ "rew" ] )

	def filters( self ):
		result = [ ]
		if self.v_deinterlace.get( ) == 1: result += [ "filter:deinterlace" ]
		if self.v_reverse.get( ) == 1: result += [ "filter:clip", "in=-1", "out=0" ]
		if self.v_visualise.get( ) == 1: result += [ "filter:visualise" ]
		if self.v_greyscale.get( ) == 1: result += [ "filter:chroma" ]
		if self.v_thread.get( ) == 1: result += [ "filter:threader" ]
		result += [ "filter:conform" ]
		return result

	def load( self ):
		files = tkFileDialog.askopenfilenames( )
		if len( files ) > 0:
			self.execute( [ "stop", "clear", "purge" ] )
			list = [ ]
			for f in files: 
				list += [ f ]
			self.execute( list + [ "filter:playlist", "slots=%d" % len( list ) ] + self.filters( ) + [ "start" ] )

	def append( self ):
		files = tkFileDialog.askopenfilenames( )
		if len( files ) > 0:
			list = [ ]
			for f in files: 
				list += [ f ]
			self.execute( list + [ "filter:playlist", "slots=%d" % len( list ) ] + self.filters( ) + [ "start" ] )

	def next( self ):
		self.execute( [ "stop", "play" ] )

	def shutdown( self ):
		self.execute( [ "shutdown" ] )
		sys.exit( 0 )

	def scrub( self, value ):
		if self.b_scale_pressed:
			self.execute( [ str( value ), "seek" ] )

	def scale_up( self, event ):
		self.b_scale_pressed = False

	def scale_down( self, event ):
		self.b_scale_pressed = True

	def status_cb( self, position, length ):
		if not self.b_scale_pressed:
			if position != -1 and length != -1:
				self.b_scale.config( from_=0, to=length )
				self.b_scale.set( position )
			else:
				tkMessageBox.showerror( "jbatch_ui", "Server communication dropped - client exiting" )
				sys.exit( 0 )

def usage( ):
	print """
jbatch_ui [ options ]

Options:

    -h, --host=host : hostname to access (default: localhost)
    -p, --port=port : port number to use (default: 45537)

Special:

    --help          : this information
"""

def main( ):
	switches = { }
	switches[ "host" ] = "localhost"
	switches[ "port" ] = "45537"
	next = None

	for i in sys.argv[ 1: ]:
		arg = None

		if i.startswith( "--" ) and i.find( "=" ) > 0:
			i, arg = i.split( "=", 1 )

		if i in ( "-p", "--port" ):
			next = "port"
		elif i in ( "-h", "--host" ):
			next = "host"
		elif i in ( "--help" ):
			usage( )
			sys.exit( 0 )
		else:
			arg = i

		if arg is not None:
			if next is not None:
				switches[ next ] = arg
				next = None

	root = Tkinter.Tk( )
	try:
		app = App( root, switches[ "host" ], int( switches[ "port" ] ) )
		root.mainloop( )
	except Exception, e:
		print "Server not running at " + switches[ "host" ] + ":" + switches[ "port" ] + "."

if __name__ == "__main__":
	main( )

