import openmedialib
import openpluginlib
import balance
import common
import properties
import operands
import os
import sys

class stack_exception( Exception ):
	def __init__( self, arg ):
		Exception.__init__( self, arg )

class stack:
	def __init__( self ):
		self.objects = { }
		self.stack = [ ]
		self.dict = { }
		self.dict[ "purge" ] = self.purge
		self.dict[ "clear" ] = self.clear
		self.dict[ "swap" ] = self.swap
		self.dict[ "2swap" ] = self.swap2
		self.dict[ "rot" ] = self.rot
		self.dict[ "drop" ] = self.drop
		self.dict[ "2drop" ] = self.drop2
		self.dict[ "dup" ] = self.dup
		self.dict[ "debug" ] = self.debug
		self.dict[ "decap" ] = self.decap
		self.dict[ "clone" ] = self.clone
		self.dict[ "over" ] = self.over
		self.dict[ "render" ] = self.render
		self.dict[ "nip" ] = self.nip
		self.dict[ "tuck" ] = self.tuck
		self.dict[ "!" ] = self.assign
		self.dict[ "decap!" ] = self.assign_decap
		self.dict[ ":" ] = self.start
		self.dict[ ";" ] = self.end
		self.dict[ "@" ] = self.deref
		self.dict[ "play" ] = self.play
		self.dict[ "store" ] = self.store
		self.dict[ "store!" ] = self.assign_store
		self.dict[ "stack" ] = self.show_stack
		self.dict[ "vars" ] = self.vars
		self.dict[ "functions" ] = self.show_functions
		self.dict[ "+" ] = self.add
		self.dict[ "-" ] = self.sub
		self.dict[ "*" ] = self.mult
		self.dict[ "/" ] = self.div
		self.functions = { }
		self.defining = False
		self.definition = [ ]
		self.locals = [ { } ]

	def purge( self ):
		self.objects = { }

	def is_image_extension( self, extension ):
		all_extensions = ""

		oil_plugins = openpluginlib.discovery( openpluginlib.all_query_traits( "openimagelib", "", "", 0 ) )
		for y in oil_plugins:
			all_extensions = all_extensions + y.in_filter( )

 		extension = extension.lower( )
 		return all_extensions.find( extension ) != -1

	def withdraw( self, op ):
		result = None
		if op.input is None:
			uri = op.uri
			if not self.objects.has_key( uri ):
				self.objects[ uri ] = [ ]
			instances = self.objects[ uri ]
			if len( instances ) > 0:
				result = instances.pop( )
			elif uri.find( "var:" ) == 0:
				result = operands.input_operand( unicode( uri[ 4: ] ), op )
			elif uri.find( "op:" ) == 0:
				result = operands.input_operand( unicode( uri[ 3: ] ), op )
			elif uri == "filter:wrapper":
				result = common.input_wrapper( )
			elif uri.find( "filter:" ) == 0:
				result = openmedialib.create_filter( str( uri[ 7: ] ) )
			else:
				# Special case for images
				real = str( uri )
				path, name = os.path.split( uri )
				prefix, ext = os.path.splitext( name )
				if ext != "" and self.is_image_extension( ext ):
					real = str( real ) + "/sequence:"
				result = openmedialib.create_input( str( real ).replace( '\\', '/' ) )
			if result is not None:
				properties.update_prop( result, "rpn_uri", str( uri ) )
				result.use_count = 0
			else:
				result = operands.input_operand( unicode( uri ), op )
				result.use_count = 0
		else:
			result = op.input
		return result

	def deposit( self, input ):
		if input is not None and input.use_count == 0:
			i = 0
			while i < input.slots( ):
				self.deposit( input.fetch_slot( i ) )
				input.connect( None, i )
				i += 1
			if input.reuse( ):
				uri = properties.retrieve_uri( input )
				properties.update_prop( input, "rpn_uri", str( uri ) )
				self.objects[ uri ].append( input )
		else:
			input.use_count -= 1

	def clear( self ):
		while not self.empty( ):
			tos = self.pop( )
			self.deposit( tos )

	def swap( self ):
		tos = self.pop( )
		bos = self.pop( )
		self.push_input( tos )
		self.push_input( bos )

	def swap2( self ):
		i1 = self.pop( )
		i2 = self.pop( )
		i3 = self.pop( )
		i4 = self.pop( )
		self.push_input( i2 )
		self.push_input( i1 )
		self.push_input( i4 )
		self.push_input( i3 )

	def rot( self ):
		tos = self.pop( )
		bos = self.pop( )
		uos = self.pop( )
		self.push_input( bos )
		self.push_input( tos )
		self.push_input( uos )

	def dup( self ):
		tos = self.pop( )
		tos.use_count += 1
		self.push_input( tos )
		self.push_input( tos )

	def drop( self ):
		tos = self.pop( )
		self.deposit( tos )

	def drop2( self ):
		self.drop( )
		self.drop( )

	def decap( self ):
		tos = self.pop( )
		if tos is not None:
			i = 0
			while i < tos.slots( ):
				self.push_input( tos.fetch_slot( i ) )
				tos.connect( None, i )
				i += 1
			self.deposit( tos )
		return tos

	def clone_node( self, node ):
		if node is not None:
			i = 0
			while i < node.slots( ):
				self.clone_node( node.fetch_slot( i ) )
				i += 1
			self.push( properties.retrieve_uri( node ) )
			self.push_args( properties.value_prop( properties.find_prop( node, "rpn_props" ) ) )
			self.push_callback( node.fetch_callback( ) )

	def clone( self ):
		tos = self.pop( )
		self.push_input( tos )
		self.clone_node( tos )

	def over( self ):
		tos = self.pop( )
		bos = self.pop( )
		self.push_input( bos )
		self.push_input( tos )
		self.clone_node( bos )

	def nip( self ):
		self.swap( )
		self.drop( )

	def tuck( self ):
		self.clone( )
		tos = self.pop( )
		self.swap( )
		self.push_input( tos )

	def assign( self ):
		op = self.pop( )
		tos = self.pop( )
		tos.use_count += 1
		uri = op.uri
		if uri.find( "var:" ) == 0: uri = uri[ 4: ]
		self.locals[ -1 ][ uri ] = tos

	def deref( self ):
		op = self.pop( )
		uri = op.uri
		if uri.find( "var:" ) == 0: uri = uri[ 4: ]
		if uri in self.locals[ -1 ].keys( ):
			self.push_input( self.locals[ -1 ][ uri ] )
		else:
			raise stack_exception( "Variable " + op.uri + " not defined." )

	def assign_decap( self ):
		op = self.stack.pop( )
		tos = self.decap( )
		self.locals[ -1 ][ op.uri ] = tos

	def start( self ):
		self.defining = True

	def end( self ):
		if self.defining == True:
			name = self.definition.pop( 0 )
			self.functions[ name ] = [ ] + self.definition
			self.definition = [ ]
			self.defining = False
		else:
			raise stack_exception( "Syntax error: function terminator out of place" )

	def play( self ):
		input = self.pop( )
		if input is not None:
			common.play( input )
			self.deposit( input )

	def store( self ):
		op = self.pop( )
		tos = self.pop( )
		common.play( tos, op.uri, op.operand.properties )
		self.deposit( tos )

	def assign_store( self ):
		var = self.stack.pop( )
		op = self.pop( )
		tos = self.pop( )
		common.play( tos, op.uri, op.operand.properties )
		self.deposit( tos )
		self.push( op.uri )
		self.stack.append( var )
		self.assign( )

	def add( self ):
		val1 = self.pop( )
		val0 = self.pop( )
		self.push( str( int( val0 ) + int( val1 ) ) )

	def sub( self ):
		val1 = self.pop( )
		val0 = self.pop( )
		self.push( str( int( val0 ) - int( val1 ) ) )

	def mult( self ):
		val1 = self.pop( )
		val0 = self.pop( )
		self.push( str( int( val0 ) * int( val1 ) ) )

	def div( self ):
		val1 = self.pop( )
		val0 = self.pop( )
		self.push( str( int( val0 ) / int( val1 ) ) )

	def push( self, id ):
		result = True
		if self.defining == True and id != ";":
			self.definition.append( id )
		elif id.startswith( "sdl." ):
			# Ugh - temporary access to store properties
			split = id.split( "=" )
			ksplit = split[ 0 ].split( "." )
			if common.preview.video_store is None:
				common.preview.sdl[ ksplit[ 1 ] ] = split[ 1 ]
			else:
				common.assign_prop( common.preview.video_store.properties( ).get_property( ksplit[ 1 ] ), [ ksplit[ 1 ], split[ 1 ] ] )
		elif id.endswith( ".oml" ):
			self.push_file( id )
		elif id in self.functions.keys( ):
			self.locals.append( { } )
			self.push_args( self.functions[ id ] )
			self.locals.pop( -1 )
		elif id in self.dict.keys( ):
			self.dict[ id ]( )
		elif id in self.locals[ -1 ].keys( ):
			self.push_input( self.locals[ -1 ][ id ] )
		elif id.find( "/sequence:" ) > 0:
			self.stack.append( operands.operand( id ) )
		else:
			if id.find( "=" ) > 0:
				if id.find( "=var:" ) > 0:
					split = id.split( "=var:" )
					split[ 1 ] = self.locals[ -1 ][ split[ 1 ] ].uri
					id = split[ 0 ] + "=" + str( split[ 1 ] )
				self.stack[ -1 ].properties.append( id )
			elif id != "":
				self.stack.append( operands.operand( id ) )
			else:
				result = False
		return result

	def push_file( self, file ):
		if not os.path.exists( file ):
			path = sys.path[ 0 ]
			temp = os.path.join( os.path.abspath( path ), file )
			if os.path.exists( temp ):
				file = temp
		if os.path.exists( file ):
			file = os.path.abspath( file )
			root = os.path.dirname( file )
			f = open( file, "r" )
			if f is not None:
				for line in f:
					tokens = line.strip( ).split( " " )
					result = []
					current = ""
					for token in tokens:
						if current == "":
							current = token
						else:
							current += " " + token
						if not current.startswith( "\"" ) or current.endswith( "\"" ):
							if current.startswith( "\"" ) and current.endswith( "\"" ):
								current = current[ 1:-1 ]
							if current != "" and not os.path.isabs( current ) and os.path.exists( os.path.join( root, current ) ):
								current = os.path.join( root, current )
							if current != "":
								result.append( current )
							current = ""
					if current != "":
						raise stack_exception( "Malformed " + file + ": " + line )
					if len( result ) > 0:
						self.push_args( result )
				f.close( )
		else:
			raise stack_exception( "Unable to locate " + file )

	def push_callback( self, callback ):
		result = True
		if len( self.stack ) > 0:
			self.stack[ -1 ].callback = callback
		else:
			result = False
		return result

	def push_args( self, properties ):
		result = True
		if properties is not None:
			for i in properties: 
				if i.startswith( "#" ): break
				result &= self.push( i )
		return result

	def push_input( self, input ):
		id = properties.retrieve_uri( input )
		self.stack.append( operands.operand( id, input ) )
		self.push_args( properties.value_prop( properties.find_prop( input, "rpn_props" ) ) )
		self.push_callback( input.fetch_callback( ) )
		return True

	def pop( self ):
		result = None
		if len( self.stack ) > 0:
			op = self.stack.pop( )
			result = self.withdraw( op )
			if result is not None:
				op.assign( result )
				slot = result.slots( )
				while slot > 0:
					slot -= 1
					if result.fetch_slot( slot ) is not None:
						self.push_input( result.fetch_slot( slot ) )
					result.connect( self.pop( ), slot )
				op.assign( result )
				properties.update_prop( result, "rpn_length", result.get_frames( ) )
				properties.update_prop( result, "rpn_props", [] + op.properties )
				result.register( op.callback )
			else:
				result = operands.input_operand( op.uri, op )
		else:
			raise stack_exception( "Stack underflow" )
		return result

	def empty( self ):
		return len( self.stack ) == 0

	def debug( self, title = "stack trace" ):
		print title
		for i in self.stack:
			print str( i )
		print

	def render( self, title = "top of stack" ):
		print title
		tos = self.pop( )
		if tos is not None:
			balance.walk( tos )
			self.push_input( tos )
		else:
			print "<empty>"

	def show_stack( self, title = "current stack" ):
		print title
		tos = self.pop( )
		if tos is not None:
			balance.stack( tos )
			self.push_input( tos )
			print
		else:
			print "<empty>"

	def vars( self, title = "variables" ):
		print title
		for frame in self.locals:
			for i in frame.keys( ):
				print i + ": " + str( frame[ i ] )
			print

	def show_functions( self, title = "functions" ):
		print title
		for key in self.functions.keys( ):
			print ":", key
			for word in self.functions[ key ]:
				print word,
			print
			print ";"
			print
